@@ -103,13 +103,15 @@ import logging
 import re
 
 try:
-    from peewee import CharField
+    from tortoise import Model
+    from tortoise.exceptions import DoesNotExist, IntegrityError
+    from tortoise.fields import TextField
 except ImportError:
-    CharField = None
+    Model = TextField = None
 
 import immp
 from immp.hook.command import command
-from immp.hook.database import BaseModel, DatabaseHook
+from immp.hook.database import AsyncDatabaseHook
 from immp.hook.identity import IdentityProvider
 
 
@@ -136,9 +138,9 @@ def _emoji_replace(match):
     return " " if left and right else ""
 
 
-if CharField:
+if Model:
 
-    class SyncBackRef(BaseModel):
+    class SyncBackRef(Model):
         """
         One of a set of references, each pointing to a representation of a source message.
 
@@ -153,17 +155,17 @@ if CharField:
                 Message ID as generated by the plug.
         """
 
-        key = CharField()
-        network = CharField()
-        channel = CharField()
-        message = CharField()
+        key = TextField()
+        network = TextField()
+        channel = TextField()
+        message = TextField()
 
         class Meta:
             # Unique constraint for each message.
-            indexes = ((("network", "channel", "message"), True),)
+            unique_together = (("network", "channel", "message"),)
 
         @classmethod
-        def map_from_sent(cls, sent):
+        async def map_from_sent(cls, sent):
             """
             Take a :class:`.Receipt` and attempt to resolve it to a key from a previously synced
             reference, then defer to :meth:`map_from_key` to make the mapping with the key.
@@ -177,16 +179,16 @@ if CharField:
                     Generated reference mapping.
             """
             try:
-                key = cls.select(cls.key).where(cls.network == sent.channel.plug.network_id,
-                                                cls.channel == sent.channel.source,
-                                                cls.message == str(sent.id)).get().key
-            except cls.DoesNotExist:
+                ref = await cls.filter(network=sent.channel.plug.network_id,
+                                       channel=sent.channel.source,
+                                       message=str(sent.id)).get()
+            except DoesNotExist:
                 raise KeyError
             else:
-                return cls.map_from_key(key)
+                return await cls.map_from_key(ref.key)
 
         @classmethod
-        def map_from_key(cls, key):
+        async def map_from_key(cls, key):
             """
             For a given key, fetch all messages references synced from the corresponding source, and
             group them by network and channel.
@@ -199,7 +201,7 @@ if CharField:
                 ((str, str), .SyncBackRef) dict:
                     Generated reference mapping.
             """
-            backrefs = cls.select().where(cls.key == key)
+            backrefs = await cls.filter(key=key)
             if not backrefs:
                 raise KeyError
             mapped = defaultdict(list)
@@ -298,7 +300,7 @@ class SyncCache:
         # Reverse mapping, Channel -> ID -> SyncRef.
         self._lookup = defaultdict(dict)
 
-    def add(self, ref, back=False):
+    async def add(self, ref, back=False):
         """
         Add a :class:`.SyncRef` to the cache.  This will also commit a new :class:`.SyncBackRef` to
         the database if configured.
@@ -315,17 +317,18 @@ class SyncCache:
                 The same ref, useful for shorthand add-then-return.
         """
         self._cache[ref.key] = ref
-        data = []
         for channel, ids in ref.ids.items():
             for id_ in ids:
                 self._lookup[channel][id_] = ref.key
-                data.append({"key": ref.key, "network": channel.plug.network_id,
-                             "channel": channel.source, "message": id_})
-        if self._hook.db and not back:
-            SyncBackRef.insert_many(data).on_conflict("ignore").execute()
+                if self._hook._db and not back:
+                    try:
+                        await SyncBackRef.create(key=ref.key, network=channel.plug.network_id,
+                                                 channel=channel.source, message=id_)
+                    except IntegrityError:
+                        pass
         return ref
 
-    def __getitem__(self, key):
+    async def get(self, key):
         if isinstance(key, immp.Receipt) and key.channel.plug == self._hook.plug:
             # Message from the sync channel itself, so just look the key up directly.
             key = key.id
@@ -335,22 +338,22 @@ class SyncCache:
                 # Use the existing cache entry as-is (entry in _lookup <=> entry in _cache).
                 return self._cache[self._lookup[key.channel][key.id]]
             except KeyError:
-                if not self._hook.db:
+                if not self._hook._db:
                     raise
             # Not cached locally, but the database is configured: check there for a reference,
             # build a new SyncRef with an empty source, and cache it.
-            key, mapped = SyncBackRef.map_from_sent(key)
-            return self.add(SyncRef.from_backref_map(key, mapped, self._hook.host), True)
+            key, mapped = await SyncBackRef.map_from_sent(key)
+            return await self.add(SyncRef.from_backref_map(key, mapped, self._hook.host), True)
         elif isinstance(key, str):
             # As above, check the local cache directly first.
             try:
                 return self._cache[key]
             except KeyError:
-                if not self._hook.db:
+                if not self._hook._db:
                     raise
             # Now check the database for the key.
-            _, mapped = SyncBackRef.map_from_key(key)
-            return self.add(SyncRef.from_backref_map(key, mapped, self._hook.host), True)
+            _, mapped = await SyncBackRef.map_from_key(key)
+            return await self.add(SyncRef.from_backref_map(key, mapped, self._hook.host), True)
         else:
             raise TypeError(key)
 
@@ -455,7 +458,7 @@ class SyncPlug(immp.Plug):
             raise immp.PlugError("Send to unknown sync channel: {}".format(repr(channel)))
 
     async def delete(self, sent):
-        await self._hook.delete(self._hook._cache[sent.id])
+        await self._hook.delete(await self._hook._cache.get(sent.id))
 
 
 class _SyncHookBase(immp.Hook):
@@ -479,14 +482,14 @@ class _SyncHookBase(immp.Hook):
             return False
         return True
 
-    def _replace_recurse(self, msg, func, *args):
+    async def _replace_recurse(self, msg, func, *args):
         # Switch out entire messages for copies or replacements.
         if msg.reply_to:
-            msg.reply_to = func(msg.reply_to, *args)
+            msg.reply_to = await func(msg.reply_to, *args)
         attachments = []
         for attach in msg.attachments:
             if isinstance(attach, immp.Message):
-                attachments.append(func(attach, *args))
+                attachments.append(await func(attach, *args))
             else:
                 attachments.append(attach)
         msg.attachments = attachments
@@ -608,7 +611,6 @@ class SyncHook(_SyncHookBase):
 
     def __init__(self, name, config, host):
         super().__init__(name, config, host)
-        self.db = None
         # Message cache, stores IDs of all synced messages by channel.
         self._cache = SyncCache(self)
         # Hook lock, to put a hold on retrieving messages whilst a send is in progress.
@@ -622,14 +624,17 @@ class SyncHook(_SyncHookBase):
                 host.add_channel(label, immp.Channel(self.plug, label))
         else:
             self.plug = None
+        self._db = False
 
-    @property
-    def channels(self):
+    def on_load(self):
         try:
-            return {virtual: [self.host.channels[label] for label in labels]
-                    for virtual, labels in self.config["channels"].items()}
-        except KeyError as e:
-            raise immp.ConfigError("No channel {} on host".format(repr(e.args[0]))) from None
+            self.host.resources[AsyncDatabaseHook].add_models(SyncBackRef)
+        except KeyError:
+            self._db = False
+        else:
+            self._db = True
+
+    channels = immp.ConfigProperty({None: [immp.Channel]})
 
     def label_for_channel(self, channel):
         labels = []
@@ -643,17 +648,6 @@ class SyncHook(_SyncHookBase):
         else:
             return labels[0]
 
-    async def start(self):
-        try:
-            self.db = self.host.resources[DatabaseHook].db
-        except KeyError:
-            pass
-        else:
-            self.db.create_tables([SyncBackRef], safe=True)
-
-    async def stop(self):
-        self.db = None
-
     def _test(self, channel, user):
         return any(channel in channels for channels in self.channels.values())
 
@@ -705,7 +699,7 @@ class SyncHook(_SyncHookBase):
 
     async def send(self, label, msg, origin=None, ref=None):
         """
-        Send a message to all channels in this sync.
+        Send a message to all channels in this synced group.
 
         Args:
             label (str):
@@ -731,7 +725,7 @@ class SyncHook(_SyncHookBase):
                 log.debug("Skipping already-synced target channel %r: %r", synced, ref)
                 continue
             local = base.clone()
-            self._replace_recurse(local, self._replace_ref, synced)
+            await self._replace_recurse(local, self._replace_ref, synced)
             await self._alter_recurse(local, self._alter_identities, synced)
             await self._alter_recurse(local, self._alter_name)
             queue.append(self._send(synced, local))
@@ -744,7 +738,7 @@ class SyncHook(_SyncHookBase):
                 ref.ids.update(ids)
             else:
                 ref = SyncRef(ids, source=msg, origin=origin)
-            self._cache.add(ref)
+            await self._cache.add(ref)
         # Push a copy of the message to the sync channel, if running.
         if self.plug:
             sent = immp.SentMessage(id_=ref.key, channel=immp.Channel(self.plug, label),
@@ -763,7 +757,7 @@ class SyncHook(_SyncHookBase):
         if queue:
             await gather(*queue)
 
-    def _replace_ref(self, msg, channel):
+    async def _replace_ref(self, msg, channel):
         if not isinstance(msg, immp.Receipt):
             log.debug("Not replacing non-receipt message: %r", msg)
             return msg
@@ -773,7 +767,7 @@ class SyncHook(_SyncHookBase):
                                 reply_to=msg.reply_to, joined=msg.joined, left=msg.left,
                                 title=msg.title, attachments=msg.attachments, raw=msg.raw)
         try:
-            ref = self._cache[msg]
+            ref = await self._cache.get(msg)
         except KeyError:
             log.debug("No match for source message: %r", msg)
             return base
@@ -804,7 +798,7 @@ class SyncHook(_SyncHookBase):
             pass
         ref = None
         try:
-            ref = self._cache[sent]
+            ref = await self._cache.get(sent)
         except KeyError:
             if sent.deleted:
                 log.debug("Ignoring deleted message not in sync cache: %r", sent)
@@ -834,30 +828,35 @@ class ForwardHook(_SyncHookBase):
     Hook to propagate messages from a source channel to one or more destination channels.
     """
 
-    schema = immp.Schema({immp.Optional("users"): immp.Nullable([str])}, _SyncHookBase.schema)
+    schema = immp.Schema({immp.Optional("users"): immp.Nullable([str]),
+                          immp.Optional("groups", dict): {str: [str]}}, _SyncHookBase.schema)
 
-    @property
-    def _channels(self):
-        try:
-            return {self.host.channels[key]: tuple(self.host.channels[label] for label in value)
-                    for key, value in self.config["channels"].items()}
-        except KeyError as e:
-            raise immp.ConfigError("No such channel '{}'".format(repr(e.args[0])))
+    _channels = immp.ConfigProperty({immp.Channel: [immp.Channel]})
+    _groups = immp.ConfigProperty({immp.Group: [immp.Channel]})
 
-    async def send(self, channel, msg):
+    async def _targets(self, channel):
+        targets = set()
+        if channel in self._channels:
+            targets.update(self._channels[channel])
+        for group, channels in self._groups.items():
+            if await group.has_channel(channel):
+                targets.update(channels)
+        return targets
+
+    async def send(self, msg, channels):
         """
-        Send a message to all channels in this sync.
+        Send a message to all channels in this forwarding group.
 
         Args:
-            channel (.Channel):
-                Source channel that defines the underlying forwarding channels to send to.
             msg (.Message):
                 External message to push.
+            channels (.Channel list):
+                Set of target channels to forward the message to.
         """
         queue = []
         clone = msg.clone()
         await self._alter_recurse(clone, self._alter_name)
-        for synced in self._channels[channel]:
+        for synced in channels:
             local = clone.clone()
             await self._alter_recurse(local, self._alter_identities, synced)
             queue.append(self._send(synced, local))
@@ -875,5 +874,8 @@ class ForwardHook(_SyncHookBase):
 
     async def on_receive(self, sent, source, primary):
         await super().on_receive(sent, source, primary)
-        if primary and sent.channel in self._channels and self._accept(source):
-            await self.send(sent.channel, source)
+        if not primary or not self._accept(source):
+            return
+        targets = await self._targets(sent.channel)
+        if targets:
+            await self.send(source, targets)
