@@ -534,10 +534,16 @@ class TelegramFile(immp.File):
             .TelegramFile:
                 Parsed file object.
         """
-        file = await telegram._api("getFile", _Schema.file, params={"file_id": id_})
-        url = ("https://api.telegram.org/file/bot{}/{}"
-               .format(telegram.config["token"], file["file_path"]))
-        return immp.File(name, type_, url)
+        try:
+            file_ = await telegram._api("getFile", _Schema.file, params={"file_id": id_})
+        except TelegramAPIRequestError:
+            # Can happen if the file is too big, in which case just return a placeholder.
+            log.warning("Failed to retrieve message attachment", exc_info=True)
+            return immp.File(name, type_)
+        else:
+            url = ("https://api.telegram.org/file/bot{}/{}"
+                   .format(telegram.config["token"], file_["file_path"]))
+            return immp.File(name, type_, url)
 
 
 class TelegramMessage(immp.Message):
@@ -1071,6 +1077,9 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             return self._users[id_]
         try:
             data = await self._client(tl.functions.users.GetFullUserRequest(int(id_)))
+        except ValueError:
+            log.warning("Missing entity for user %d", id_)
+            return None
         except BadRequestError:
             return None
         user = TelegramUser.from_proto_user(self, data.user)
@@ -1213,6 +1222,9 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
                         users += [TelegramUser.from_proto_user(self, user) for user in data.users]
                     else:
                         break
+            except ValueError:
+                log.warning("Missing entity for channel %d", chat)
+                return None
             except BadRequestError:
                 return None
             else:
@@ -1221,6 +1233,9 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             chat = abs(int(channel.source))
             try:
                 data = await self._client(tl.functions.messages.GetFullChatRequest(chat))
+            except ValueError:
+                log.warning("Missing entity for channel %d", chat)
+                return None
             except BadRequestError:
                 return None
             else:
